package no.ntnu.idatt1002.demo.data.Economics;

import no.ntnu.idatt1002.demo.data.Budget.BudgetItem;
import no.ntnu.idatt1002.demo.data.Budget.GeneralBudget;

/**
 * Class for getting an overview
 * on your total revenue.
 *
 * @author andreas
 */
public class Overview {
    private final IncomeRegister incomeRegister;
    private final ExpenseRegister expenseRegister;
    private double balance;
    private GeneralBudget budget;
    /**
     * Class constructor that creates an empty incomeRegister, expenseRegister
     * and set totalRevenue to zero.
     */
    public Overview(){
        this.incomeRegister = new IncomeRegister();
        this.expenseRegister = new ExpenseRegister();
        this.balance = 0;
    }

    /**
     * Class constructor that takes in an incomeRegister and
     * expenseRegister and calculates total balance
     * based on them.
     * @param incomeRegister the incomeRegister you want to overview.
     * @param expenseRegister the expenseRegister you want to overview.
     */
    public Overview(IncomeRegister incomeRegister, ExpenseRegister expenseRegister){
        this.incomeRegister = incomeRegister;
        this.expenseRegister = expenseRegister;
        updateBalance();
    }

    /**
     * Get the total revenue of overview
     * @return the overview´s total revenue
     */
    public double getTotalBalance(){
        return balance;
    }

    /**
     * Method for updating the total revenue
     * by taking the total sum of the income register
     * minus the total sum of the expense register.
     */
    public void updateBalance(){
        balance = incomeRegister.getTotalSum() - expenseRegister.getTotalSum();
    }

    /**
     * Method for getting a percentage
     * of the total revenue
     * @param percentage the percentage you want to get
     * @return a percentage of total revenue
     */
    public double getPercentageOfTotalBalance(double percentage){
        return Math.round(balance*(percentage/100));
    }

    /**
     * Method for getting Expense subtracted from
     * a BudgetItem for a specific ExpenseCategory.
     *
     * @param category the ExpenseCategory you want to use.
     * @return Expense subtracted from BudgetItem for category.
     */
    public double getBudgetItemMinusExpense(ExpenseCategory category){
        if(budget.hasBudgetCategory(category)){
            return budget.getBudgetItem(category).getBudgetAmount() - expenseRegister.getExpenseByCategory(category).getTotalSum();
        } else{
            throw new IllegalArgumentException("There is no Budget for the budget category");
        }
    }

    /**
     * Method for checking if you have used up
     * a specific BudgetItem.
     *
     * @param category the ExpenseCategory you want to check.
     * @return true or false depending on if you have used it up.
     */
    public boolean hasUsedUpBudgetItem(ExpenseCategory category){
        return (getBudgetItemMinusExpense(category)<=0);
    }


}
